package net.technics;

import java.util.HashMap;
import java.util.Map;
/**
 * Singleton permettant de verrouiller des objets dans des listes de verrous
 * Les objets peuvent être classés par types
 * @author jc
 *
 */
public class LockEngine {
	private static LockEngine instance;
	private Map<Object, LockList> lockLists;
	
	private LockEngine(){
		lockLists=new HashMap<>();
	}
	
	/**
	 * Retourne l'instance de moteur
	 * @return instance de LockEngine
	 */
	public static LockEngine getInstance(){
		if(instance==null)
			synchronized (LockEngine.class) {
				instance=new LockEngine();
			}
		return instance;
	}
	/**
	 * Crée et retourne une nouvelle liste de verrous de clé keyList
	 * @param keyList clé de la liste de verrous
	 * @return la liste créée
	 */
	public static LockList createNewLockList(Object keyList){
		return createNewLockList(keyList, LockList.defaultTTL);
	}
	
	/**
	 * Crée et retourne une nouvelle liste de verrous de clé keyList avec un ttl
	 * @param keyList clé de la liste de verrous
	 * @param ttl durée de vie maximale d'un verrou avant déverrouillage automatique
	 * @return
	 */
	public static LockList createNewLockList(Object keyList,int ttl){
		System.out.println("Création de LockList <<keyList>> : "+keyList+" <<ttl>> : "+ttl);
		Map<Object, LockList> list=getInstance().lockLists;
		LockList ll=new LockList(ttl);
		list.put(keyList, ll);
		return ll;
	}
	/**
	 * Vide une liste de verrous de clé keyList
	 * @param keyList clé de la liste
	 */
	public static void clear(Object keyList){
		LockList list=getInstance().lockLists.get(keyList);
		if(list!=null)
			list.clear();
	}
	/**
	 * Vide toutes les listes créées
	 */
	public static void clear(){
		LockEngine engine=getInstance();
		for(Map.Entry<Object, LockList> entry:engine.lockLists.entrySet()){
			entry.getValue().clear();
		}
	}

	/**
	 * Retourne une liste de verrous, l'instancie si elle n'existe pas
	 * @param keyList clé de la liste
	 * @return la liste de clé keyList
	 */
	public static LockList getLockList(Object keyList){
		LockList list=getInstance().lockLists.get(keyList);
		if(list==null){
			list=createNewLockList(keyList);
		}
		return list;
	}
	
	/**
	 * Essaye de vérouiller le verrou de clé keyObject dans la liste de clé keyList
	 * @param keyList clé de la liste
	 * @param keyObject clé du verrou
	 * @return retourne vrai si le verrouillage a été réalisé (pas d'autre verrou déjà posé)
	 */
	public static boolean lock(Object keyList,String keyObject){
		return getLockList(keyList).lock(keyObject);
	}
	/**
	 * Dévérouille le verrou de clé keyObject dans la liste de clé keyList
	 * @param keyList clé de la liste
	 * @param keyObject clé du verrou
	 * @return retourne vrai si le déverrouillage a été effectif (verrou existant)
	 */
	public static boolean unlock(Object keyList,String keyObject){
		return getLockList(keyList).unlock(keyObject);
	}
	
	/**
	 * Retourne le verrou de clé keyObject présent dans la liste de clé keyList
	 * @param keyList clé de la liste
	 * @param keyObject clé du verrou
	 * @return le verrou trouvé ou null s'il n'existe pas
	 */
	public static Lock getLock(Object keyList,String keyObject){
		return getLockList(keyList).get(keyObject);
	}
	
	@Override
	  public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException();
	  }
	
}
